@@ -2,9 +2,6 @@
 from .InterfaceObjects import ASG,StopCondition
 #import helper functions
 from .HelperFunctions import *
-#import the KB_Interface
-from Interface_KB import KB_Interface
-
 
 #-----------------------------------------------------------------------------------------------------------
 #
@@ -14,9 +11,7 @@ from Interface_KB import KB_Interface
 #               - Discuss: Do we need to provide the rules in the getASG()
 #               - Discuss: Do we need to provide the rules in the setASG()
 #               - Discuss: Can we assume that the assembly system is in place when performing setASG()
-#               - Discuss: Do we need seperate functions to UPDATE and ADD stopcriterions?
 #               - Discuss: Do we need seperate functions to UPDATE and ADD DFA rules?
-#               - ERROR: new classes are not generated (+linked) correctly -> subcomposition of platformRoot does not work
 #
 # -----------------------------------------------------------------------------------------------------------
 
@@ -33,7 +28,6 @@ def getASG(AssemblySystemName,model,KBPath):
     """
 
     found = 0
-    #ASG = []        #[Name, Description,processingType,[operationGeneratorName,operationGeneratorType],[operationSelectorName,operationSelectorType],[operationEvaluatorName,operationEvaluatorType],[terminatorName,[terminators]]]]       with terminators = [name,description,value,stopCriterion enum]
     InterfaceObject = ASG(None) #interface object placeholder
     if len(model.includesAssemblySystem.items) != 0:
         #match the assemblySystem by Name
@@ -46,7 +40,7 @@ def getASG(AssemblySystemName,model,KBPath):
                 InterfaceObject.Description = ASG_temp.hasDescription
                 InterfaceObject.ProcessingType = ASG_temp.hasProcessingType.name
 
-                # try-catch structure to fetch the ASG GENERATOR
+                # -- ASG GENERATOR --
                 try:
                         InterfaceObject.Generator=[ASG_temp.hasSinglePartGenerator.hasName, 'SinglePartGenerator']
                 except:
@@ -76,15 +70,11 @@ def getASG(AssemblySystemName,model,KBPath):
                 except:
                     x = "No Rule selector available"
 
-
-
+                # -- Terminator --
                 terms = []
                 try:
                     for terminator in ASG_temp.hasTerminator.hasStopCriterion.items:
-                        #initialize
-                        termList = []
-
-                        #fetch the termintator name
+                        # -- stopconditions --
                         s = StopCondition(None, terminator.hasName, terminator.hasDescription,terminator.hasValue, terminator.hasStopCriterion.name)
 
                         #add to criteria
@@ -96,8 +86,6 @@ def getASG(AssemblySystemName,model,KBPath):
             else:
                 x = 1
 
-
-
         if found:
             return InterfaceObject
         else:
@@ -125,12 +113,12 @@ def updateASG(AssemblySystemName,interfaceObject,model,KBPath):
         for assemblySystem in model.includesAssemblySystem.items:
             if AssemblySystemName == assemblySystem.hasName:
                 found=1
-                #fetch ASG and setup ASG return
+                # -- ASG setup --
                 assemblySystem.hasAssemblyAlgorithmConfiguration.hasName = interfaceObject.Name
                 assemblySystem.hasAssemblyAlgorithmConfiguration.hasDescription = interfaceObject.Description
                 assemblySystem.hasAssemblyAlgorithmConfiguration.hasProcessingType = interfaceObject.ProcessingType
 
-                # try-catch structure to fetch the ASG GENERATOR
+                # -- ASG GENERATOR --
                 try:
                     assemblySystem.hasAssemblyAlgorithmConfigurationhasSinglePartGenerator.hasSinglePartGenerator.hasName = interfaceObject.Generator[0]
                 except:
@@ -140,7 +128,7 @@ def updateASG(AssemblySystemName,interfaceObject,model,KBPath):
                 except:
                     x = "No MultiPartGenerator available"
 
-                # try-catch structure to fetch the ASG SELECTOR
+                # -- ASG SELECTOR --
                 try:
                     assemblySystem.hasAssemblyAlgorithmConfigurationhasSinglePartGenerator.hasDefaultSelector.hasName =  interfaceObject.Selector[0]
                 except:
@@ -151,7 +139,7 @@ def updateASG(AssemblySystemName,interfaceObject,model,KBPath):
                 except:
                     x = "No Rule selector available"
 
-                # try-catch structure to fetch the ASG EVALUATOR
+                # -- ASG EVALUATOR --
                 try:
                     assemblySystem.hasAssemblyAlgorithmConfigurationhasSinglePartGenerator.hasUniformEvaluator.hasName = interfaceObject.Evaluator[0]
                 except:
@@ -164,7 +152,7 @@ def updateASG(AssemblySystemName,interfaceObject,model,KBPath):
 
 
 
-                terms = []
+                # -- Terminator container --
                 for terminator in assemblySystem.hasAssemblyAlgorithmConfiguration.hasTerminator.hasStopCriterion.items:
                     for stp in interfaceObject.StopConditionList:
                         if stp.Name==terminator.hasName:
@@ -197,95 +185,83 @@ def setASG( AssemblySystemName, InterfaceObject,model,KBPath,path_ecore,MM,API):
 
           :return int Error: -1 = error, 1= function performed correcly
     """
-    # --!HACK!--:check if an Optimization class already exists, if not, generate a placeholder
-    if len(model.includesAssemblySystem.items) == 0:
-        model = EmptyClassHack(API=API, elementName="AssemblySystem")
 
-    API.initialize(API.ECORE_path)
-    #create new AssemblySystem to overwrite the old one
-    AS = API.create("AssemblySystem")
+    # -- create new AssemblySystem --
+    AS = API.create_connected("AssemblySystem")
     AS.hasName = AssemblySystemName
-    #generate a new ASG config and configure
+    # -- create new ASG --
     ASG_ = API.create("AssemblyAlgorithmConfiguration")
-    #ASG_ = API.create_custom(parent=assemblySystem,e_class="AssemblyAlgorithmConfiguration")
-    # ASG attributes
     ASG_.hasName = InterfaceObject.Name
     ASG_.hasDescription = InterfaceObject.Description
     #ASG.hasProcessingType = InterfaceObject.ProcessingType                 #TODO: check how to set this
 
     if 'SinglePartGenerator' in InterfaceObject.Generator[1]:
-        # try-catch structure to fetch the ASG GENERATOR
+        # -- ASG GENERATOR --
         try:
             ASG_.hasSinglePartGenerator.hasName = InterfaceObject.Generator[0]
         except:
-            x = "No SinglePartGenerator available"       #TODO: does not work -> generate new class fails!
-            #create new class if not existing
+            x = "No SinglePartGenerator available"
+            # -- create new class if not existing --
             g_ = API.create("SinglePartGenerator")
-            #g_ = API.create_custom(ASG_,"SinglePartGenerator")
             g_.hasName = InterfaceObject.Generator[0]
             ASG_.hasSinglePartGenerator = g_
 
     if 'MultiPartGenerator' in InterfaceObject.Generator[1]:
-        # try-catch structure to fetch the ASG GENERATOR
+        # -- ASG GENERATOR --
         try:
             ASG_.hasMultiPartGenerator.hasName = InterfaceObject.Generator[0]
         except:
-            x = "No MultiPartGenerator available"       #TODO: does not work -> generate new class fails!
-            #create new class if not existing
+            x = "No MultiPartGenerator available"
+            # -- create new class if not existing --
             g_ = API.create("MultiPartGenerator")
-            #g_ = API.create_custom(ASG_,"MultiPartGenerator")
             g_.hasName = InterfaceObject.Generator[0]
             ASG_.hasMultiPartGenerator = g_
 
     if 'DefaultSelector' in InterfaceObject.Selector[1]:
-        # try-catch structure to fetch the ASG GENERATOR
+        # -- ASG SELECTOR --
         try:
             ASG_.hasDefaultSelector.hasName = InterfaceObject.Selector[0]
         except:
-            x = "No Default selector available"       #TODO: does not work -> generate new class fails!
-            #create new class if not existing
+            x = "No Default selector available"
+            # -- create new class if not existing --
             g_ = API.create("DefaultSelector")
-            #g_ = API.create_custom(ASG_,"DefaultSelector")
             g_.hasName = InterfaceObject.Selector[0]
             ASG_.hasDefaultSelector = g_
 
     if 'RuleSelector' in InterfaceObject.Selector[1]:
-        # try-catch structure to fetch the ASG GENERATOR
+        # -- ASG SELECTOR --
         try:
             ASG_.hasRuleSelector.hasName = InterfaceObject.Selector[0]
         except:
-            x = "No RuleSelector available"       #TODO: does not work -> generate new class fails!
-            #create new class if not existing
+            x = "No RuleSelector available"
+            # -- create new class if not existing --
             g_ = API.create("RuleSelector")
-            #g_ = API.create_custom(ASG_, "RuleSelector")
             g_.hasName = InterfaceObject.Selector[0]
             ASG_.hasRuleSelector = g_
 
     if 'UniformEvaluator' in InterfaceObject.Evaluator[1]:
-        # try-catch structure to fetch the ASG GENERATOR
+        # -- ASG EVALUATOR --
         try:
             ASG_.hasUniformEvaluator.hasName = InterfaceObject.Evaluator[0]
         except:
-            x = "No uniform evaluator available"       #TODO: does not work -> generate new class fails!
-            #create new class if not existing
+            x = "No uniform evaluator available"
+            # -- create new class if not existing --
             g_ = API.create("UniformEvaluator")
-            #g_ = API.create_custom(ASG_, "UniformEvaluator")
             g_.hasName = InterfaceObject.Evaluator[0]
             ASG_.hasDefaultSelector = g_
 
     if 'RuleEvaluator' in InterfaceObject.Evaluator[1]:
-        # try-catch structure to fetch the ASG GENERATOR
+        # -- ASG EVALUATOR --
         try:
             ASG_.hasRuleEvaluator.hasName = InterfaceObject.Evaluator[0]
         except:
-            x = "No RuleEvaluator available"       #TODO: does not work -> generate new class fails!
-            #create new class if not existing
+            x = "No RuleEvaluator available"
+            # -- create new class if not existing --
             g_ = API.create("RuleEvaluator")
-            #g_ = API.create_custom(ASG_, "RuleEvaluator")
             g_.hasName = InterfaceObject.Evaluator[0]
             ASG_.hasRuleEvaluator = g_
 
-    #stop conditions
+    # -- stop conditions --
     terminator_container = API.create("Terminator")
     terminator_container.hasName = "Default"
     ASG_.hasTerminator = terminator_container
@@ -298,6 +274,6 @@ def setASG( AssemblySystemName, InterfaceObject,model,KBPath,path_ecore,MM,API):
         #terminator.hasStopCriterion = stp.StopCriteria         #TODO:assigns currently a string!
         terminator_container.hasStopCriterion.append(terminator)
 
-    AS.hasAssemblyAlgorithmConfiguration = ASG_             #TODO: not added to the root -  not in output file
+    AS.hasAssemblyAlgorithmConfiguration = ASG_
 
     updateKBv6(KBPath,model)
\ No newline at end of file
