@@ -275,7 +275,6 @@ class KB_Interface():
             if interfaceObject.Name ==self.model.includesProduct.items[0].hasProductName:
                 self.model.includesProduct.items[0].hasProductName=interfaceObject.Name
                 self.model.includesProduct.items[0].hasSTEPfile=interfaceObject.STEPFile
-            #   TODO: add the parameters-> discuss which parameters
 
                 self.updateKBv6()
                 return 1
@@ -301,16 +300,13 @@ class KB_Interface():
             return -1
         else:
             # --!HACK!--:check if an Optimization class already exists, if not, generate a placeholder
-            self.model = EmptyClassHack(API=self, elementName="Product")
+            #self.model = EmptyClassHack(API=self, elementName="Product")
 
-            self.initialize(self.ECORE_path)
-            product = self.create("Product")
+            product = self.create_connected("Product")
             product.hasProductName = interfaceObject.Name
             product.hasSTEPfile = interfaceObject.STEPFile
             # TODO: discuss what is set via the setProduct function e.g. do we already set the analysis model?
 
-
-            #self.model.includesProduct.items.append(product)
             self.updateKBv6()
             return 1
 
@@ -348,7 +344,7 @@ class KB_Interface():
                         part.partList.append(p)
 
                     #link the contactFeatures
-                    #TODO: not yet implemented
+                    #TODO: not yet implemented - No example available to validate
 
                     #material
                     try:
@@ -379,7 +375,6 @@ class KB_Interface():
                     productPart.hasPartType=interfaceObject.ProductType
                     #productPart.Quantity = interfaceObject.Quantity                 # TODO:Quantity not in MM v6
 
-
                     # update the parameters
                     for par in productPart.hasModel.hasModelParameter.items:
                         for par_new in interfaceObject.parameterList:
@@ -397,7 +392,7 @@ class KB_Interface():
 
 
                     # link the contactFeatures
-                    # TODO: not yet implemented
+                    #TODO: not yet implemented - No example available to validate
 
                     # material
                     productPart.hasMaterial.hasName=interfaceObject.material[0]
@@ -413,7 +408,7 @@ class KB_Interface():
         else:
             return -1  # no ProductPart instance
 
-    def setProductPart(self, interfaceObject,triggerFault = False):
+    def setProductPart(self, interfaceObject):
         """
              Function to add a ProductPart model to an blank KB .
              The ProductPart interface object is used to interface with the function.
@@ -423,13 +418,9 @@ class KB_Interface():
 
              :return int Error: -1 = error, 1= function performed correcly
         """
-        # --!HACK!--:check if an Optimization class already exists, if not, generate a placeholder
-        if len(self.model.includesProductPart.items) == 0:
-            self.model = EmptyClassHack(API=self, elementName="ProductPart")
 
-        self.initialize(self.ECORE_path)
         #setup a eCore class of a productPart
-        part = self.create("ProductPart")
+        part = self.create_connected("ProductPart")
 
         part.hasPartName = interfaceObject.Name
         # productPart.hasDescription = interfaceObject.Description       # TODO:Quantity not in MM v6
@@ -438,44 +429,40 @@ class KB_Interface():
 
 
         #add model to ProductPart
-        model = self.create("Model")
+        model = self.create_connected("Model")
         model.hasModelType = ""
         part.hasModel = model
-        # update the parameters
 
+        # update the parameters
         for par_new in interfaceObject.parameterList:
-            parameter = self.create("Parameter")
+            parameter = self.create_connected("Parameter")
             parameter.hasKey = par_new.Key
             parameter.hasValue = par_new.Value
             part.hasModel.hasModelParameter.append(parameter)
 
-        # link the product parts - Recursive!
-        #for par in productPart.hasCompositionEdge.items:
-        #    for par_new in interfaceObject.partList:
-        #        if par_new.Name == par.hasPartName:
-        #            # TODO:update the subproduct
-        #            error = self.updateProductPart(par_new)
+        # link the product parts - Recursive! # TODO:update the subproduct - No example available to validate
+        for par_new in interfaceObject.partList:
+            if par_new.Name == part.hasPartName:
+                error = self.setProductPart(par_new)
+                part.hasCompositionEdges.append(self.model.includesProductPart.items[-1])
 
         # link the contactFeatures
-        # TODO: not yet implemented
+        # TODO: not yet implemented - No example available to validate
 
         # material
-        material = self.create("Material")
+        material = self.create_connected("Material")
         part.hasMaterial = material
         part.hasMaterial.hasName = interfaceObject.material[0]
         part.hasMaterial.hasCost = interfaceObject.material[1]
         part.hasMaterial.hasDamping = interfaceObject.material[2]
         part.hasMaterial.hasDensity = interfaceObject.material[3]
         part.hasMaterial.hasTransparency = interfaceObject.material[4]
-        stiffness = self.create("Stiffness")
+        stiffness = self.create_connected("Stiffness")
         part.hasMaterial.hasStiffness = stiffness
         part.hasMaterial.hasStiffness.hasScalarMatrix6x6 = interfaceObject.material[5]
 
-        #--DEBUG PyECORE--
-        if triggerFault:
-            self.model.includesProduct[0].hasProductPart.append(part)
-        #----------------
-
+        self.model.includesProduct[-1].hasProductPart.append(part)
+        
         self.updateKBv6()
 
         return 1
@@ -544,8 +531,6 @@ class KB_Interface():
 # --------------------------------------------------------------------------------------------------------------------------------------------
 #
 #   Assembly Sequence
-#           #TODO:update the functions (get,update,set) -> allSequences AND Sequence('Name')
-#           #TODO: implemented via references instead of composition -> can we change this to allign with other modeled parts?
 #           #TODO: assembly contains lots of components without an ID or name (e.g. orientation, geometry), can we adapt this to enable re-use?
 #           #TODO: hasFastner links to productPart, is that correct?
 # ---------------------------------------------------------------------------------------------------------------------------------------------
@@ -591,6 +576,7 @@ class KB_Interface():
 
               :return int Error: -1 = error, 1= function performed correcly
         """
+        #TODO: discuss if this is required
         x=10
         return -1
 
@@ -606,41 +592,39 @@ class KB_Interface():
 
              :return int Error: -1 = error, 1= function performed correcly
         """
-        self.initialize(self.ECORE_path)
-        AssemblySequence = self.create("AssemblySequence")
-
-        #----add AssemblyMetric----- TODO: single metric for now, ok?
-        asm = self.create("AssemblyMetric")
-        asm.hasName=InterfaceObject.AssemblyMetric[0]
-        asm.hasValue = InterfaceObject.AssemblyMetric[1]
-        AssemblySequence.hasAssemblyMetric.append(asm)
-
-        # -- recursive AssemblyOptions --
-        AssemblySequence.hasAssemblyOption = self.setAssemblyOption_NEW(Option=InterfaceObject.AssemblyOptions)
-
         #-----------------------------------------------------------------------------------------------------
         #   DEBUG PURPOSE: Add product and assemblySystem if not exist
         #-----------------------------------------------------------------------------------------------------
         if len(self.model.includesAssemblySystem.items)==0:
 
             # ---------------------- PRODUCT DUMMY ---------------------------
-            product = self.create("Product")
+            product = self.create_connected("Product")
             product.hasProductName = "DUMMY-PRODUCT-DEBUG"
             product.hasSTEPfile = 'c:/NEW/PATH/TO/THE/STEPFILE'
 
-
             # ---------------------- ASSEMBLY SYSTEM DUMMY ---------------------------
-            asys = self.create("AssemblySystem")
+            asys = self.create_connected("AssemblySystem")
 
             # -- minimal dependecies --
             asys.hasName = AssemblySystemName
             asys.hasGUID = ""
-            GravityDirection = self.create("Direction")
+            GravityDirection = self.create_connected("Direction")
+            GravityDirection.hasRx=1.0
+            GravityDirection.hasRy = 2.0
+            GravityDirection.hasRz = 3.0
+            GravityDirection.hasX = 4.0
+            GravityDirection.hasY = 5.0
+            GravityDirection.hasZ = 6.0
             asys.hasGravityDirection = GravityDirection
 
-            GroundPosition = self.create("Position")
+            GroundPosition = self.create_connected("Position")
+            GroundPosition.hasX = 0.1
+            GroundPosition.hasY = 0.2
+            GroundPosition.hasZ = 0.3
             asys.hasGroundPosition = GroundPosition
 
+            product.hasAssemblySystem.append(asys)
+
         else:
             # ---------------------- PRODUCT REAL ---------------------------
             product = self.model.includesProduct.items[0]
@@ -650,6 +634,20 @@ class KB_Interface():
             for a in self.model.includesAssemblySystem.items:
                 if a.hasName == AssemblySystemName:
                     asys = a
+        # -----------------------------------------------------------------------------------------------------
+        #   Create assembly sequence
+        # -----------------------------------------------------------------------------------------------------
+        AssemblySequence = self.create_connected("AssemblySequence")
+
+        #----add AssemblyMetric-----
+        asm = self.create_connected("AssemblyMetric")
+        asm.hasName=InterfaceObject.AssemblyMetric[0]
+        asm.hasValue = InterfaceObject.AssemblyMetric[1]
+        AssemblySequence.hasAssemblyMetric.append(asm)
+
+        # -- recursive AssemblyOptions --
+        AssemblySequence.hasAssemblyOption = self.setAssemblyOption_NEW(Option=InterfaceObject.AssemblyOptions)
+
 
         asys.hasAssemblySequence = AssemblySequence
         product.hasAssemblySystem.append(asys)
@@ -719,15 +717,15 @@ class KB_Interface():
         try:
 
             # -- define the assemblyOption --
-            ASO = self.create("SubAssembly")
+            ASO = self.create_connected("SubAssembly")
             ASO.hasName = Option.Name
 
             # operations
             for operation in Option.Operations:
-                op = self.create("Operation")
+                op = self.create_connected("Operation")
 
                 # ----add OperationMetric-----
-                om = self.create("OperationMetric")
+                om = self.create_connected("OperationMetric")
                 om.hasName = operation.OperationMetric[0]
                 om.hasValue = operation.OperationMetric[1]
                 op.hasOperationMetric.append(om)
@@ -740,17 +738,17 @@ class KB_Interface():
 
                 # ----add Tools-----
                 for tool in operation.Tool:
-                    t = self.create("Tool")
+                    t = self.create_connected("Tool")
                     t.hasName = tool.Name
                     t.hasCost = tool.Cost
                     t.hasMass = tool.Mass
 
                     # --# ----Geometry-----
-                    g = self.create("Geometry")
+                    g = self.create_connected("Geometry")
                     g.hasDataInStepFile = tool.Geometry
                     t.hasGeometry = g
                     # --# ----Orientation-----
-                    o = self.create("Orientation")
+                    o = self.create_connected("Orientation")
                     o.hasData = tool.Orientation
                     t.hasOrientation = o
 
@@ -760,8 +758,6 @@ class KB_Interface():
                     ASO.hasOperation.append(op)
 
 
-
-
             #Linking the recursive operations!
             subAssemblyOptions = []
             for recursive_option in Option.RecursiveAssemblyOption:
@@ -775,59 +771,6 @@ class KB_Interface():
         except:
             return []
 
-
-    def setAssemblyOption(self, AssemblyOption):
-        #non-recursive for now, TODO:EXTEND
-        try:
-            aso = self.create("SubAssembly")
-            aso.hasName = AssemblyOption.Name
-            #aso.hasDescription = AssemblyOption.Description
-
-            # --- add assembly operation ---
-            op = self.create("Operation")
-            # ----add Fastners----
-                #TODO: productParts by name?
-            # ----add OperationMetric-----
-            om = self.create("OperationMetric")
-            om.hasName = AssemblyOption.OperatorMetric[0]
-            om.hasValue = AssemblyOption.OperatorMetric[1]
-            op.hasOperationMetric.append(om)
-
-            # ----add Operators-----
-                #TODO:implement after hacks are removed!
-            # ----add Tools-----
-            for tool in AssemblyOption.ToolList:
-                t = self.create("Tool")
-                t.hasName = tool.Name
-                t.hasCost = tool.Cost
-                t.hasMass = tool.Mass
-
-                #--# ----Geometry-----
-                g = self.create("Geometry")
-                g.hasDataInStepFile = tool.Geometry
-                t.hasGeometry=g
-                # --# ----Orientation-----
-                o = self.create("Orientation")
-                o.hasData = tool.Orientation
-                t.hasOrientation=o
-
-                op.hasTool.append(t)
-
-            aso.hasOperation.append(op)
-
-            # --- add the recursive operations ---
-            for recursive_option in AssemblyOption.RecursiveAssemblyOption:
-                temp = self.setAssemblyOption(AssemblyOption=recursive_option)
-                AssemblyOption.hasRecursiveAssemblyOption.append(temp)
-
-            return aso
-
-        except:
-
-            if self.DEBUG:print("ERROR - No option added")
-            return -1
-
-
 # -----------------------------------------------------------------------------------------------------------
 #
 #   Contract-Ontology extension
@@ -944,6 +887,13 @@ class KB_Interface():
         self.model = importInstanceModel(path_ecore, path_instance, VERBOSE=False)
         return self.model
 
+    def importInstanceModel_NEW(self, path_ecore, path_instance, VERBOSE=False):
+        """
+              For internal use only!
+            """
+        self.MM,self.model,self.model_instance = importInstanceModel_NEW(path_ecore, path_instance, VERBOSE=False)
+        return self.MM,self.model,self.model_instance
+
     def addXMLResource(self, rset):
         """
               For internal use only!
@@ -968,6 +918,12 @@ class KB_Interface():
             """
         return create(e_class, self.MM, self.model)
 
+    def create_connected(self, e_class):
+        """
+              For internal use only!
+            """
+        return create_connected(e_class, self.MM, self.model)
+
     def create_custom(self, parent, e_class):
         """
               For internal use only!
@@ -1014,14 +970,12 @@ class KB_Interface():
                         print("Performance Model already added - Do we need to update them?")
                         old = an_old
                         new = an_new
-                        # self.updateAnalysis(old,new)
 
                 if found == 0:  # no matching productPart found -> add?
                     print("Performance Model not found - Do we need to add them?")
                     part = self.createAnalysis(an_new)
                     self.model.includesAnalysis.items.append(part)
 
-
         else:
             for an_new in AnalysisList:  # assuming that no performance models are added yet
                 part = self.createAnalysis(an_new)
